function [Irep,a,b,c,d,Area,algorithm]=EMG(Iexp,E,expvar,csel)

%--------------------------------------------------------------
% GPA program by Santiago Cavanillas and J.M. Daz
% December 2014 
%
% Exponential Modified Gaussian Peak Adjustment.
%
% ffunction [Irep,a,b,c,d,Area,algorithm]=EMG(Iexp,E,expvar,csel)
%--------------------------------------------------------------

%checking dimensions

[nexpvar,nothing]=size(expvar);
[nexp,npot]=size(Iexp);
[nrow,ncomp]=size(csel);

if nexp>npot,Iexp=Iexp';
	disp('There are more voltammograms than potentials. Matrix has been transposed')
	[nexp,npot]=size(Iexp);
end;%(it assumes that usually there are more potentials than voltammograms)

[npot1,nothing]=size(E);
exist1=csel(1,:);%beginning of existence range
exist2=csel(2,:);%end of existence range

exist=ones(nexp,ncomp);

for i=1:nexp;
    for k=1:ncomp;
        if exist1(1,k)>i | exist2(1,k)<i;
           exist(i,k)=0;
        end
    end
end

if nrow~=2, error('Error in the dimensions of csel');end
if npot1~=npot, error('Iexp and E dimensions must agree');end
if nexpvar~=nexp, error('Iexp and expvar dimensions must agree');end

%normalized variable number of column (Quitar??)
for i=1:npot,x(i,1)=i/npot;end

%normalization of Iexp and esv (Iexpn and v)
maxI=max(max(Iexp));
Iexpn=Iexp./maxI;

%%%%%%%%%%%%%%%%%%%%%%%%%%%    Repetition    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
agree='n';
while agree=='n'|agree=='N';
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Estimation of b

    plot(E,Iexpn,'r');xlabel('t / s');ylabel('normalized current');
    
    if nargin>4, title('You must introduce the components in this order: M, ML, ML2... ');
    else title('select the top of every peak with the mouse and, when finished, press enter'); 
    end
    
    [b0,current0]=ginput;

% number of peaks, ncomp1

    [ncomp1,nothing]=size(current0);
    if ncomp~=ncomp1, error('The number of component must agree');end

% Estimation of a

    for n=1:ncomp;
        est0(:,n)=current0(n).*exp(-(E-b0(n)).^2./0.0025);
    end;

    for n=1:ncomp;
        
    plot(E,Iexpn,'k',E,est0(:,n),'r',E,current0(n)./2*ones(1,npot),'r');
    
    title('select the left side in the W1/2 of the red component with the mouse and, when finished')
    [Eleft(n),y1]=ginput(1);
    
    title('select the right in the W1/2 of the red component with the mouse and, when finished')
    [Eright(n),y2]=ginput(1);

    w0(n,1)=abs(Eleft(n)-Eright(n));
    a0(n,1)=w0(n,1)*current0(n,1);
    
    end;

%%%%%%%%%%%%%%%%%%Determination of the required algorithm%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    Epeak=b0;
    alg=zeros(1,ncomp);

    for n=1:ncomp
    difleft(n)=(abs(Eleft(n)))-(abs(Epeak(n)));
    difright(n)=(abs(Eright(n)))-(abs(Epeak(n)));
    reldif(n)=abs(difleft(n)/difright(n));
    end 

    for n=1:ncomp,
        if reldif(n) <= 0.8, alg(n)=1; end
        if reldif(n) >= 1.2, alg(n)=2; end
    end
    
    algorithm=alg;
    
%%%%%%%% construction of the initial value matrix with c and d %%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    Ev=[E,zeros(npot,ncomp-1)];
    parameter00=[a0,b0,0.05*ones(ncomp,1),0.005*ones(ncomp,1)]';

    minE=min(E);maxE=max(E);

    for n=1:ncomp;
        
        Esv=[1,zeros(1,ncomp-1);alg;Ev];
        options=optimset('Display','off','Diagnostics','off','MaxIter',100,'TolX',0.0001,'TolFun',0.0001);
        
        for k=1:ncomp
            if alg==0
            lv(:,k)=[0;minE;0;0];uv=[1;maxE;2000;2000];
            else lv(:,k)=[0;minE;0;0];uv=[1;maxE;0.25;0.25]; end
        end
        
        [parameter0]=lsqcurvefit('EMGcalc',parameter00,Esv,est0(:,n),lv,uv,options);
    
        a00(n,:)=negtozero(parameter0(1,:));
        b00(n,:)=parameter0(2,:);
        c0(n,:)=parameter0(3,:);
        d0(n,:)=parameter0(4,:);
    end

    parameter0=[a0,b0,c0(:,1),d0(:,1)]';

    for n=1:ncomp;
        parameterrep0=[];
        parameterrep0=[a0(n,1);b0(n,1);c0(n,1);d0(n,1)];
        Esvrep0=[1,zeros(1,ncomp-1);alg;Ev];
        [Irepn0(n,:)]=EMGcalc(parameterrep0,Esvrep0);
    end

    plot(E,Irepn0,'r',E,est0,'k');title('Reproduced matrix');ylabel('Current / A');xlabel('E / V');disp('press any key');pause;


% Fitting of exponential modified gaussian parameters %

    for i=1:nexp;
        
    Esv=[exist(i,:);alg;Ev];
    
		for k=1:ncomp,%takes advantage of the previously fitted parameters
			if exist(i,k)==1 & i>exist1(1,k),parameter0(:,k)=[a(i-1,k),b(i-1,k),c(i-1,k),d(i-1,k)];end
        end
        
    options=optimset('Display','off','Diagnostics','off','MaxIter',100,'TolX',0.000001,'TolFun',0.000001);
    lv=[];uv=[];
    
        for k=1:ncomp
            
            if alg==0
                 lv(:,k)=[0;minE;0;0];uv=[1;maxE;2000;2000];
            else lv(:,k)=[0;minE;0;0];uv=[1;maxE;0.25;0.25]; 
            end
            
        end
        
        [parameter]=lsqcurvefit('EMGcalc',parameter0,Esv,Iexpn(i,:)',lv,uv,options);
    
        a(i,:)=negtozero(parameter(1,:));
        b(i,:)=parameter(2,:);
        c(i,:)=parameter(3,:);
        d(i,:)=parameter(4,:);
    
        for k=1:ncomp,%forcing again zero concentration in the regions were the peak does not exist
            if exist(i,k)==0, a(i,k)=0;c(i,k)=0;d(i,k)=0; 
            end 
        end
    end

% Plot of gaussian parameters %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    subplot(4,1,1);plot(expvar,a,'o-');xlabel('experimental variable');ylabel('Area ');
    title('fitted a');
    subplot(4,1,2);plot(expvar,b,'o-');xlabel('experimental variable');ylabel('E ');
    title('fitted b');
    subplot(4,1,3);plot(expvar,c,'o-');xlabel('experimental variable');ylabel('Related width ');
    title('fitted c');
    subplot(4,1,4);plot(expvar,d,'o-');xlabel('experimental variable');ylabel('Related symmetry ');
    title('fitted d');

% reproduced (Irep) and error (Ierr) matrices 

    for i=1:nexp;
    parameterrep=[];
    parameterrep=[a(i,:);b(i,:);c(i,:);d(i,:)];
	Esvrep=[exist(i,:);alg;Ev];
    [Irepn(i,:)]=EMGcalc(parameterrep,Esvrep);
    end

    Irep=Irepn*maxI;
    figure;plot(E,Irep,'r',E,Iexp,'k');title('Reproduced matrix');ylabel('Current / A');xlabel('E / V');pause,
    
    Area=[];
    for n=1:ncomp,
        if alg(n)==0,
            for i=1:nexp,
                for k=1:ncomp,
                    if  exist(i,k)==1,
                        Area(i,k)=(0.5*a(i,k)*pi^0.5*(((1/d(i,k))^0.5)+((1/c(i,k))^0.5)))*maxI;
                    else Area(i,k)=0;
                    end
                end
            end
        else Area(:,n)=a(:,n).*maxI;
        end
    end
    
    plot(expvar,Area);title('Is the calculated areas O.K. (y/n) ?');ylabel('Areas / a.u.');xlabel('Experimental variables');
    agree=input('Is the calculated areas O.K. (y/n) ','s');
   
end

% computing lack of fit

disp('lack of fit, in %')

lof=sqrt(sum((Irep-Iexp).^2)/sum(Iexp.^2))*100,


    



